@@ -6,7 +6,7 @@ package Mite::Project;
 use Mite::Miteception;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.004000';
+our $VERSION   = '0.005000';
 
 has sources =>
   is            => ro,
@@ -91,9 +91,11 @@ sub inject_mite_functions {
 
     my $source = $self->source_for( $file );
     my $pkg    = $source->class_for( $package, $kind eq 'role' ? 'Mite::Role' : 'Mite::Class' );
+    $pkg->shim_name( $shim );
 
     no strict 'refs';
     ${ $package .'::USES_MITE' } = ref( $pkg );
+    ${ $package .'::MITE_SHIM' } = ref( $shim );
 
     my $has = sub {
         my ( $names, %args ) = @_;
@@ -152,6 +154,11 @@ sub inject_mite_functions {
         return;
     } if $kind eq 'class' && $requested->( 'extends', 1 );
 
+    *{ $package .'::requires' } = sub {
+        $pkg->add_required_methods( @_ );
+        return;
+    } if $kind eq 'role' && $requested->( 'requires', 1 );
+
     for my $modifier ( qw( before after around ) ) {
         *{ $package .'::'. $modifier } = sub {
             my ( $names, $coderef ) = &$parse_mm_args;
@@ -160,6 +167,7 @@ sub inject_mite_functions {
                 or Carp::croak( "Expected a coderef method modifier" );
             ArrayRef->of(Str)->check( $names ) && @$names
                 or Carp::croak( "Expected a list of method names to modify" );
+            $pkg->add_required_methods( @$names );
             return;
         } if $requested->( $modifier, 1 );
     }
