@@ -6,7 +6,7 @@ package Mite::Role;
 use Mite::Miteception;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.004000';
+our $VERSION   = '0.005000';
 
 use Path::Tiny;
 use B ();
@@ -25,6 +25,10 @@ has name =>
   isa           => Str,
   required      => true;
 
+has shim_name =>
+  is            => rw,
+  isa           => Str;
+
 has source =>
   is            => rw,
   isa           => InstanceOf['Mite::Source'],
@@ -33,16 +37,34 @@ has source =>
 
 has roles =>
   is            => ro,
-  isa           => ArrayRef,
+  isa           => ArrayRef[Object],
   builder       => sub { [] };
 
 has constants =>
   is            => ro,
-  isa           => HashRef,
+  isa           => HashRef[Str],
   builder       => sub { {} };
 
+has required_methods =>
+  is            => ro,
+  isa           => ArrayRef[Str],
+  builder       => sub { [] };
+
 ##-
 
+sub BUILD {
+    my $self = shift;
+
+    require Type::Registry;
+    my $reg = 'Type::Registry'->for_class( $self->name );
+    $reg->add_types( 'Types::Standard' );
+    $reg->add_types( 'Types::Common::Numeric' );
+    $reg->add_types( 'Types::Common::String' );
+
+    my $library = eval { $self->project->config->data->{types} };
+    $reg->add_types( $library ) if $library;
+}
+
 sub skip_compiling {
     return false;
 }
@@ -223,6 +245,14 @@ roles are the only supported roles. Sorry.
 ERROR
 }
 
+sub add_required_methods {
+    my $self = shift;
+
+    push @{ $self->required_methods }, @_;
+
+    return;
+}
+
 sub does_list {
     my $self = shift;
     return (
@@ -339,7 +369,11 @@ sub _compile_package {
 sub _compile_uses_mite {
     my $self = shift;
 
-    return sprintf 'our $USES_MITE = %s;', B::perlstring( ref($self) );
+    my @code = sprintf 'our $USES_MITE = %s;', B::perlstring( ref($self) );
+    if ( $self->shim_name ) {
+        push @code, sprintf 'our $MITE_SHIM = %s;', B::perlstring( $self->shim_name );
+    }
+    join "\n", @code;
 }
 
 sub _compile_pragmas {
@@ -367,6 +401,7 @@ sub __META__ {
             map { "$_\::DEMOLISH" } @$linear_isa
         ],
         HAS_BUILDARGS => $class->can('BUILDARGS'),
+        HAS_FOREIGNBUILDARGS => $class->can('FOREIGNBUILDARGS'),
     };
 }
 CODE
@@ -375,10 +410,24 @@ CODE
 sub _compile_callback {
     my $self = shift;
 
-    my $role_list = join q[, ], map B::perlstring( $_->name ), @{ $self->roles };
-    my $shim = B::perlstring( eval { $self->project->config->data->{shim} } || 'Mite::Shim' );
+    my @required = @{ $self->required_methods };
+    my %uniq; undef $uniq{$_} for @required;
+    @required = sort keys %uniq;
 
-    return sprintf <<'CODE', $role_list, $shim;
+    my $role_list = join q[, ], map B::perlstring( $_->name ), @{ $self->roles };
+    my $shim = B::perlstring(
+        $self->shim_name
+        || eval { $self->project->config->data->{shim} }
+        || 'Mite::Shim'
+    );
+    my $missing_methods = '()';
+    if ( @required ) {
+        require B;
+        $missing_methods = sprintf 'grep( !$target->can($_), %s )',
+            join q[, ], map B::perlstring( $_ ), @required;
+    }
+
+    return sprintf <<'CODE', $missing_methods, $role_list, $shim;
 # Callback which classes consuming this role will call
 sub __FINALIZE_APPLICATION__ {
     my ( $me, $target, $args ) = @_;
@@ -391,9 +440,12 @@ sub __FINALIZE_APPLICATION__ {
     $CONSUMERS{$target} = 1;
 
     my $type = do { no strict 'refs'; ${"$target\::USES_MITE"} };
-    if ( $type ne 'Mite::Class' ) {
-        return;
-    }
+    return if $type ne 'Mite::Class';
+
+    my @missing_methods;
+    @missing_methods = %s
+        and require Carp
+        and Carp::croak( "$me requires $target to implement methods: " . join q[, ], @missing_methods );
 
     my @roles = ( %s );
     my %%nextargs = %%{ $args || {} };
